
// FireAlpha.java
// Andrew Davison, ad@fivedots.coe.psu.ac.th, Nov. 2021

/* An Alpha that can be fired repeatedly for
   the period specified in the constructor. It will only
   fire if it is not already running.

   Based on code by Frank Klawonn
*/

import org.jogamp.java3d.*;


public class FireAlpha
{
  private volatile boolean isRunning = false;
  private volatile Alpha alpha;


  public FireAlpha(int period)
  {
    alpha = new Alpha(1, period);
    alpha.setStartTime(Long.MAX_VALUE);   // delay start
  }  // end of FireAlpha()

  
  public Alpha getAlpha()
  {  return alpha;  }


  public boolean isRunning()
  {  return isRunning;  }


  public void fire()
  /* if (startTime + triggerTime >= currentTime) 
     then the Alpha object starts running
  */
  {
    if (!isRunning) {
      alpha.setStartTime( System.currentTimeMillis()); // start now
      detectCompletion();
    }
  }


  private void detectCompletion()
  // set isRunning to false once the alpha has finished
  {
    isRunning = true;
    int alphaPeriod = (int)alpha.getIncreasingAlphaDuration();
    new Thread(new Runnable() {
      public void run() 
      {        
        while(!alpha.finished()) {
          // System.out.println("value: " + alpha.value());
          Simple3D.pause(alphaPeriod/10);
        }
        isRunning = false;
      }
    }).start();
  }  // end of detectCompletion()


}  // end of FireAlpha class