
// ChangeVisibility.java
// Andrew Davison, ad@fivedots.coe.psu.ac.th, Nov. 2021

/* Use two FireAlpha objects to toggle between a visible
   and invisible sphere, triggered when the user types <enter>.

   Note: the sphere is centered at (0,1,0), and the "1" on the y-axis
   is visible even when the circle is opaque.
*/

import java.util.*;

import org.jogamp.java3d.*;
import org.jogamp.vecmath.*;
import org.jogamp.java3d.utils.geometry.*;


public class ChangeVisibility
{
  private static final int DELAY = 2000;


  public static void main(String[] args)
  {
    Simple3D j3d = new Simple3D("Change Color", true, true);

    /* bg --> posTG -->  sphereTG --> sphere
                          |
                          ----> invis interpolator
                          |
                          ----> vis interpolator
    */

    Appearance app = Simple3D.color("red");

    // visibility (0 = opaque; 1 = invisble)
    TransparencyAttributes tas = new TransparencyAttributes(
                    TransparencyAttributes.NICEST, 0);
    tas.setCapability(TransparencyAttributes.ALLOW_VALUE_WRITE);
    app.setTransparencyAttributes(tas);

    Sphere sphere = new Sphere(0.5f, app);
    boolean isVisible = true;

    TransformGroup sphereTG = new TransformGroup();
    sphereTG.addChild(sphere);


    // visibility changers
    FireAlpha invisAlpha = new FireAlpha(DELAY);
    TransparencyInterpolator invis = 
              new TransparencyInterpolator(invisAlpha.getAlpha(), 
                            app.getTransparencyAttributes(), 0, 1f);
    invis.setSchedulingBounds(Simple3D.INFINITE_BOUNDS);
    sphereTG.addChild(invis);

    FireAlpha visAlpha = new FireAlpha(DELAY);
    TransparencyInterpolator vis = 
              new TransparencyInterpolator(visAlpha.getAlpha(), 
                            app.getTransparencyAttributes(), 1f, 0);
    vis.setSchedulingBounds(Simple3D.INFINITE_BOUNDS);
    sphereTG.addChild(vis);

    // position the sphere
    TransformGroup posTG = Simple3D.setTranslation(0,1,0);
    posTG.addChild(sphereTG);

    // build the scene graph
    BranchGroup bg = Simple3D.createBG(posTG);
    j3d.rootBG.addChild(bg);


    Scanner s = new Scanner(System.in);
    System.out.println("Press <enter> to toggle the sphere's visibility...");
    
    while (true) {
      s.nextLine();    // pause until user types enter

      if (isVisible) {   // && !invisAlpha.isRunning())
        invisAlpha.fire();
        System.out.println("--> invisible");
        isVisible = false;
      }
      else if (!isVisible) {  // && !visAlpha.isRunning())
        visAlpha.fire();
        System.out.println("--> visible");
        isVisible = true; 
      }
      Simple3D.pause(DELAY);  // give interpolator time to finish
    }

  }  // end of main()



}  // end of ChangeVisibility class
