
// CollisionBeh.java
// Andrew Davison, ad@fivedots.coe.psu.ac.th, Nov. 2021

/* Report collision entry and exit for a given sphere
   by printing the 'name's of the shapes involved.

   On collision entry the world coordinates of the collision
   are also printed.
*/


import java.util.*;

import org.jogamp.java3d.*;
import org.jogamp.vecmath.*;
import org.jogamp.java3d.utils.geometry.*;



public class CollisionBeh extends Behavior
{
   private WakeupCondition wakes;
   private Sphere sphere;
   private String name;   // of this sphere


   public CollisionBeh(Sphere s)
   {
     sphere = s;   
     sphere.setCollisionBounds(
           new BoundingSphere(new Point3d(0,0,0), sphere.getRadius()));
     name = sphere.getShape().getName();   
             // store the name stored in the sphere's Shape3D
     setSchedulingBounds(Simple3D.INFINITE_BOUNDS);
   }


  public void initialize()
  {
    WakeupCriterion[] crits = new WakeupCriterion[2];
    // collision entry and exit
    crits[0] = new WakeupOnCollisionEntry(sphere);
    crits[1] = new WakeupOnCollisionExit(sphere);
    wakes = new WakeupOr(crits);
    wakeupOn(wakes);
  }


  public void processStimulus(Iterator<WakeupCriterion> wCrits)
  {
    WakeupCriterion wakeUp;

    while(wCrits.hasNext()) {
      wakeUp = wCrits.next();
      if (wakeUp instanceof WakeupOnCollisionEntry) {  // collision entry
        SceneGraphPath sgp = ((WakeupOnCollisionEntry) wakeUp).
                                           getTriggeringPath();
        Node hitNode = sgp.getObject();
        if (hitNode.getName() != null) {    // to ignore unnamed shapes      
          Transform3D t3d = sgp.getTransform();
          Vector3d trans = new Vector3d();
          if (t3d != null)
            t3d.get(trans);
          System.out.println(name + " hit " + 
                             hitNode.getName() +
                             " at " + Simple3D.tupleStr(trans));
        }
      }
      else if (wakeUp instanceof WakeupOnCollisionExit) {  // collision exit
        SceneGraphPath sgp = ((WakeupOnCollisionExit) wakeUp).
                                           getTriggeringPath();
        Node hitNode = sgp.getObject();
        if (hitNode.getName() != null)
          System.out.println("  " + name + " left " +  hitNode.getName());
             // coordinates not reported on collision exit
      }
      wakeupOn(wakes);
    }
  }  // end of processStimulus()

}  // end of CollisionBeh class
