
// GasGiant.java
// Andrew Davison, ad@fivedots.coe.psu.ac.th, November 2021

/*  Use a TimeBehavior to animate a texture on a sphere

    The animation rotates the texture clockwise around 
    its center in terms of the (s,t) coordinates
    
    This requires the creation of an object so that
    the class can implement TimeUpdater. Its tick() method
    is called every DELAY ms by the Behavior.
*/

import java.util.*;

import org.jogamp.java3d.*;
import org.jogamp.vecmath.*;
import org.jogamp.java3d.utils.geometry.*;



public class GasGiant implements TimeUpdater
{
  private static final int DELAY = 50;
  private static final double TURN_ANGLE = 1;

  private BranchGroup sceneBG;

  // globals for animation
  private TextureAttributes texAttrs;
  private Transform3D toCenter, toOrigin;


  public GasGiant()
  /*   sceneBG -->  sphere
  */
  {
    // Box shape = Simple3D.texBox(1, 1, 1, "images/fire.jpg");    // easier to see texture change
    Sphere shape = Simple3D.texSphere(1, "images/fire.jpg"); 

    texAttrs = shape.getAppearance().getTextureAttributes();
    texAttrs.setCapability( TextureAttributes.ALLOW_TRANSFORM_WRITE);

    // create transforms for moving the texture so its center will be
    // the rotation point 
    toCenter = new Transform3D(); 
    toCenter.setTranslation(new Vector3d(0.5, 0.5, 0));

    toOrigin = new Transform3D(); 
    toOrigin.setTranslation(new Vector3d(-0.5, -0.5, 0));

    // build top-level scene
    sceneBG = Simple3D.createBG(shape);
  }  // end of GasGiant()


  public BranchGroup getScene()
  {  return sceneBG;  }



  public void tick(int delay)
  // animation code: rotate the texture by TURN_ANGLE
  {
    Transform3D attrT3D = new Transform3D();
    Transform3D t3d = new Transform3D();

    t3d.rotZ(Math.toRadians(TURN_ANGLE)); 
    texAttrs.getTextureTransform(attrT3D);

    // rotate around center of texture
    attrT3D.mul(toCenter);   // move origin back to center
    attrT3D.mul(t3d);          // rotate
    attrT3D.mul(toOrigin);     // move center to origin

    texAttrs.setTextureTransform(attrT3D);
  }  // end of tick()



  // -----------------------------------------


  public static void main(String[] args)
  /*
       rootBG --> sceneBG
            |
            ----> TimeBehavior
  */
  {
    Simple3D j3d = new Simple3D("Gas Giant", false, true);

    GasGiant gg = new GasGiant();
    TimeBehavior tb = new TimeBehavior(DELAY, gg);

    // build top-level scene
    BranchGroup bg = Simple3D.createBG();
    bg.addChild(gg.getScene());
    bg.addChild(tb);
    j3d.rootBG.addChild(bg);
  }  // end of main()


}  // end of GasGiant class
