
// GeometryUpdaterBeh.java
// Andrew Davison, ad@fivedots.coe.psu.ac.th, November 2021

/*
  Request an geometry update every DELAY ms.

  It is NOT possible to use Simple3D's TimeBehavior to do this since
  processStimulus() must call GeometryArray.updateData(GeometryUpdater gu)
  which will cause Java 3D to call GeometryUpdater.updateData(Geometry geo)
  defined in PulseCube.java.
*/


import java.util.*;

import org.jogamp.java3d.*;
import org.jogamp.java3d.utils.geometry.*;



public class GeometryUpdaterBeh extends Behavior
{
  private static final int DELAY = 100; // time between updates

  private WakeupCondition wakes;
  private GeometryArray ga;
  private GeometryUpdater gu;


  public GeometryUpdaterBeh(Geometry geom, GeometryUpdater gu)
  {
    ga = (GeometryArray) geom;
    this.gu = gu;
    setSchedulingBounds(Simple3D.INFINITE_BOUNDS);
  }


  public void initialize()
  {
    WakeupCriterion crits[] = new WakeupCriterion[1];
    crits[0] = new WakeupOnElapsedTime(DELAY);

    wakes = new WakeupOr(crits);
    wakeupOn(wakes);
  }  // end of initialize()


  public void processStimulus(Iterator<WakeupCriterion> wCrits)
  { 
    WakeupCriterion wakeUp;

    while(wCrits.hasNext()) {
      wakeUp = wCrits.next();
      if (wakeUp instanceof WakeupOnElapsedTime)
        ga.updateData(gu);    // request an update of the geometry 
            /* this GeometryArray.updateData(GeometryUpdater gu) will
               eventually call GeometryUpdater.updateData(Geometry geo)
               in PulseCube.java */
    }
    wakeupOn(wakes);
  }  // end of processStimulus()


} // end of GeometryUpdaterBeh class
