
// HitSpheres.java
// Andrew Davison, ad@fivedots.coe.psu.ac.th, October 2021

/* Three colored spheres (red, blue, green) orbiting the 
   y-axis centered on the origin, 
   each at a distance of 3 units, but rotating at different speeds, and
   starting at different points of the orbit path.

   A 'sun' sphere is located at the origin.

   The code reports when the red or blue spheres collide with 
   another sphere by associating a CollisionBeh object with
   each sphere.
*/


import org.jogamp.java3d.*;
import org.jogamp.vecmath.*;
import org.jogamp.java3d.utils.geometry.*;


public class HitSpheres
{

  public static void main(String[] args)
  {
    Simple3D j3d = new Simple3D("Spheres Hit", 0,2,10, false, true);
    /*
         sceneBG  ---> sunSphere
            |
            -----> rot1TG --> pos1TG --> red Sphere
            |          |
            |          ----> rotator
            |
            |----> collsionBeh for red
            |
            |
            -----> rot2TG --> pos2TG -->  blue Sphere
            |          |
            |          ----> rotator
            |
            ----> collsionBeh for blue
            |
            |
            ----> rot3TG --> pos3TG -->  green Sphere
                     |
                     ----> rotator

    */
    Sphere sunSphere = Simple3D.texSphere(1f, "images/sun.jpg");

    // orbiting red sphere
    Sphere redSphere = new Sphere(0.4f, Simple3D.color("red"));
    redSphere.getShape().setName("red");   // for collision naming
    TransformGroup pos1TG = Simple3D.setTranslation(0,0,-3);
    pos1TG.addChild(redSphere); 
    TransformGroup rot1TG = Simple3D.orbit(pos1TG, 4000);

    // detect when the red sphere collides with something
    CollisionBeh cbRed =  new CollisionBeh(redSphere);


    // orbiting blue sphere
    Sphere blueSphere = new Sphere(0.4f,  Simple3D.color("blue"));
    blueSphere.getShape().setName("blue");
    TransformGroup pos2TG = Simple3D.setTranslation(-3,0,0);
           // 
    pos2TG.addChild(blueSphere);
    TransformGroup rot2TG = Simple3D.orbit(pos2TG, 6000);  
                  // longer orbit time == slower revolution speed

    // detect when the blue sphere collides with something
    CollisionBeh cbBlue =  new CollisionBeh(blueSphere);


    // orbiting green sphere
    Sphere greenSphere = new Sphere(0.4f,  Simple3D.color("green"));
    greenSphere.getShape().setName("green");
    TransformGroup pos3TG = Simple3D.setTranslation(0,0,3);
    pos3TG.addChild(greenSphere);
    TransformGroup rot3TG = Simple3D.orbit(pos3TG, 8000);  // even slower

    // build scene
    BranchGroup sceneBG = Simple3D.createBG();
    sceneBG.addChild(sunSphere);
    sceneBG.addChild(rot1TG);
    sceneBG.addChild(cbRed);
    sceneBG.addChild(rot2TG);
    sceneBG.addChild(cbBlue);
    sceneBG.addChild(rot3TG);
    j3d.rootBG.addChild(sceneBG);
  }  // end of main()


}  // end of HitSpheres class
