
// JointedArm.java
// Andrew Davison, ad@fivedots.coe.psu.ac.th, October 2021


// import java.io.*;
// import java.util.*;

import org.jogamp.java3d.*;
import org.jogamp.vecmath.*;
import org.jogamp.java3d.utils.geometry.*;


public class JointedArm
{
  private BranchGroup armBG;
  private TransformGroup rotLower, rotUpper, rotBase;
              // the three 'joints' of the arm


  public JointedArm()
  {
    // create the shapes
    Cylinder base = new Cylinder(0.15f, 0.3f, Simple3D.color("red"));  
    Cylinder upper = new Cylinder(0.05f, 1f, Simple3D.color("green")); 
    Cylinder lower = new Cylinder(0.05f, 1f, Simple3D.color("blue")); 

    /*
         bg2 --> up23 --> rotLower --> up4 --> lower cyl
          |
          -----> up2 --> upper cyl
    */
    TransformGroup up4 = Simple3D.setTranslation(0,0.5,0);  
    up4.addChild(lower);  

    rotLower = Simple3D.createTG();
    rotLower.addChild(up4); 
    new TGViewer("rotLower", rotLower, true);  // show world coords

    TransformGroup up23 = Simple3D.setTranslation(0,1,0);  
    up23.addChild(rotLower);  

    TransformGroup up2 = Simple3D.setTranslation(0,0.5,0);  
    up2.addChild(upper);  

    BranchGroup bg2 = new BranchGroup();
    bg2.addChild(up23);
    bg2.addChild(up2);

    /*
        up1 --> rotBase --> bg1 --> rotUpper --> bg2
                             |
                             -----> base cyl
    */
    rotUpper = Simple3D.createTG();
    rotUpper.addChild(bg2); 
    new TGViewer("rotUpper", rotUpper);

    BranchGroup bg1 = new BranchGroup();
    bg1.addChild(base);
    bg1.addChild(rotUpper);

    rotBase = Simple3D.createTG();
    new TGViewer("rotBase", rotBase);
    rotBase.addChild(bg1); 

    TransformGroup up1 = Simple3D.setTranslation(0,0.15,0);  
    up1.addChild(rotBase);  


    /* complete the arm:
        armBG --> up1
    */
    armBG = Simple3D.createBG(up1);

    // a hand model (not currently used)
    BranchGroup modelBG =  Simple3D.loadModel("models/hand/hand.obj");
    TransformGroup handTG = Simple3D.setTranslation(2,0.1,0);  
    handTG.addChild(modelBG);  
    // armBG.addChild(handTG);
  }  // end of JointedArm()



  public BranchGroup getBG()
  {  return armBG;  }



  public void rotateBase(double degrees)
  // add degrees to the base's rotation around the y-axis
  {  Simple3D.rotYBy(rotBase, degrees);  }


  public void rotateUpper(double degrees)
  /* add degrees to the upper's rotation around its z-axis
     at the point where it intersects the base
  */
  {  Simple3D.rotZBy(rotUpper, degrees);  }


  public void rotateLower(double degrees)
  /* add degrees to the lower's rotation around its z-axis
     at the point where it intersects the upper cylinder
  */
  {  Simple3D.rotZBy(rotLower, degrees); }


}  // end of JointedArm class
