
// MovingModels.java
// Andrew Davison, ad@fivedots.coe.psu.ac.th, October 2021

/*  Use a DIY animation loop to create: 
      * a rotating Earth
      * a circling car
      * levitating text

    See MovingModels2.java for a TimeBehavior version.
*/

import org.jogamp.java3d.*;
import org.jogamp.vecmath.*;
import org.jogamp.java3d.utils.geometry.*;



public class MovingModels
{
  private static final int DELAY= 50;  // time between updates in ms

  private static final double CAR_RADIUS = 2.5;
  private static final double CAR_ANGLE = 5;  // angle step on each update
  private static final double EARTH_ANGLE = 5;
  private static final int MAX_LEV_STEPS = 20;


  // globals for animation
  private static TransformGroup posCarTG, rotCarTG, 
                                rotSphereTG, posTextTG;

  private static double carAngle = 180; 
  private static double levDist = 0.1;
  private static int levStep = 0;



  public static void main(String[] args)
  {  
    Simple3D j3d = new Simple3D("Moving Models", 0,1,10, true, true);
    /*
       sceneBG --> posCarTG --> rotCarTG --> car model
            |
            -----> pos2 --> rotSphereTG --> sphere
            |
            -----> posTextTG --> scale --> text
    */

    // a moveable car
    BranchGroup modelBG = Simple3D.loadModel("models/car/car.obj");
    rotCarTG = Simple3D.createTG(); 
    rotCarTG.addChild(modelBG); 
    posCarTG = Simple3D.createTG(); 
    posCarTG.addChild(rotCarTG); 

    // a rotating, positioned sphere 
    Sphere earth = Simple3D.texSphere(1, "images/earth1.jpg");
    rotSphereTG = Simple3D.createTG(); 
    rotSphereTG.addChild(earth); 
    TransformGroup pos2TG = Simple3D.setTranslation(-2,2,0);
    pos2TG.addChild(rotSphereTG);  

    // scaled, positioned text
    BranchGroup text = Simple3D.text3D("Simple3D");
    TransformGroup scaleTG = Simple3D.scale(0.3);
    scaleTG.addChild(text);  
    posTextTG = Simple3D.setTranslation(3,0.1,0);
    posTextTG.addChild(scaleTG);   

    // build top-level scene
    BranchGroup sceneBG = Simple3D.createBG();
    sceneBG.addChild(posCarTG);
    sceneBG.addChild(pos2TG);
    sceneBG.addChild(posTextTG); 

    j3d.rootBG.addChild(sceneBG);

    //  animation ------------
    while (true) {
      animate();
      Simple3D.pause(DELAY);
    }
  }  // end of main()


  private static void animate()
  {
    // rotate the sphere
    Simple3D.rotYBy(rotSphereTG, EARTH_ANGLE);  

    // drive the car in a circle
    carAngle = (carAngle + CAR_ANGLE)%360;
    double rads = Math.toRadians(carAngle);
    Simple3D.moveTo(posCarTG, 
                  CAR_RADIUS+CAR_RADIUS*Math.cos(rads), 0, 
                 -CAR_RADIUS*Math.sin(rads)); 
    Simple3D.rotYBy(rotCarTG, CAR_ANGLE); 

    // move the text up or down
    Simple3D.moveBy(posTextTG, 0, levDist, 0);
    levStep++;
    if (levStep > MAX_LEV_STEPS) {  // toggle direction
      levDist = -levDist;
      levStep = 0;
    }
  }  // end of animate()

}  // end of MovingModels class
