
// RTrans.java
// Andrew Davison, ad@fivedots.coe.psu.ac.th, October 2021

/*

 */

// import java.awt.*;

import org.jogamp.java3d.*;
import org.jogamp.vecmath.*;



public class RTrans 
{
  private static BranchGroup yellowR, redR, blueR;

  public static void main(String[] args) 
  {
    Simple3D j3d = new Simple3D("R Transformations", true, true);

    yellowR = Simple3D.text3D("R", "yellow");
    redR = Simple3D.text3D("R", "red");
    blueR = Simple3D.text3D("R", "blue");

    // BranchGroup sceneBG = posRot();
    // BranchGroup sceneBG = posScale();
    // BranchGroup sceneBG = rotScale();

    // BranchGroup sceneBG = twoDiffAxesRots();
    // BranchGroup sceneBG = twoSameAxisRots();

    // BranchGroup sceneBG = twoPosTG();
    // BranchGroup sceneBG = twoRotTG();

    BranchGroup sceneBG = posRot1TG();

    // BranchGroup sceneBG = axisAngles();

    // add everything to the universe
    j3d.rootBG.addChild(sceneBG);
  }  // end of main()



  // mixing translations, rotations, scaling


  private static BranchGroup posRot()
  // translations and rotations cannot be swapped usually
  {
    // pos then rot: rot1 --> pos1 --> red R
    TransformGroup pos1 = Simple3D.setTranslation(2, 0, 0);
    pos1.addChild(redR);
    TransformGroup rot1 = Simple3D.rotY(90);
    rot1.addChild(pos1);

    // rot then pos: pos2 --> rot2 --> blue R
    TransformGroup rot2 = Simple3D.rotY(90);
    rot2.addChild(blueR);
    TransformGroup pos2 = Simple3D.setTranslation(2, 0, 0);
    pos2.addChild(rot2);

    // build top-level scene
    BranchGroup sceneBG = new BranchGroup();
    sceneBG.addChild(yellowR); 
    sceneBG.addChild(rot1);   // red 
    sceneBG.addChild(pos2);   // blue

    return sceneBG;
  }  // end of posRot()



  private static BranchGroup posScale()
  // translations and scaling cannot be swapped usually
  {
    // pos then scale: scl1 --> pos1 --> red R
    TransformGroup pos1 = Simple3D.setTranslation(2, 0, 0);
    pos1.addChild(redR);
    TransformGroup scl1 = Simple3D.scale(2);
    scl1.addChild(pos1);

    // scale then pos: pos2 --> scl2 --> blue R
    TransformGroup scl2 = Simple3D.scale(2);
    scl2.addChild(blueR);
    TransformGroup pos2 = Simple3D.setTranslation(2, 0, 0);
    pos2.addChild(scl2);

    // build top-level scene
    BranchGroup sceneBG = new BranchGroup();
    sceneBG.addChild(yellowR);   // yellow
    sceneBG.addChild(scl1);   // red 
    sceneBG.addChild(pos2);   // blue

    return sceneBG;
  }  // end of posScale()



  private static BranchGroup rotScale()
  // rotation and scaling can be swapped usually
  {
    // rot then scale:  scl1 --> rot1 --> red R
    TransformGroup rot1 = Simple3D.rotY(90);
    rot1.addChild(redR);
    TransformGroup scl1 = Simple3D.scale(2);
    scl1.addChild(rot1);

    // scale then rot: rot2 --> scl2 --> blue R
    TransformGroup scl2 = Simple3D.scale(1.8); // smaller so is visible
    scl2.addChild(blueR);
    TransformGroup rot2 = Simple3D.rotY(90);
    rot2.addChild(scl2);

    // build top-level scene
    BranchGroup sceneBG = new BranchGroup();
    sceneBG.addChild(yellowR);   // yellow
    sceneBG.addChild(scl1);   // red 
    sceneBG.addChild(rot2);   // blue

    return sceneBG;
  }  // end of rotScale()



  // multiple translations, rotations, scalings



  private static BranchGroup twoSameAxisRots()
  // multiple rotations around the same axis are additive
  {
    // pos1 --> yellow R
    TransformGroup pos1 = Simple3D.setTranslation(2, 0, 0);
    pos1.addChild(yellowR);

    // pos then rot: rot1 --> pos2 --> red R
    TransformGroup pos2 = Simple3D.setTranslation(2, 0, 0);
    pos2.addChild(redR);
    TransformGroup rot1 = Simple3D.rotY(30);
    rot1.addChild(pos2);

    // pos then two rotations:  rot3 Y --> rot2 Y --> pos3 --> blue R
    TransformGroup pos3 = Simple3D.setTranslation(2, 0, 0);
    pos3.addChild(blueR);
    TransformGroup rot2 = Simple3D.rotY(30);
    rot2.addChild(pos3);
    TransformGroup rot3 = Simple3D.rotY(60);
    rot3.addChild(rot2);

    // build top-level scene
    BranchGroup sceneBG = new BranchGroup();
    sceneBG.addChild(pos1);   // yellow
    sceneBG.addChild(rot1);   // red 
    sceneBG.addChild(rot3);   // blue

    return sceneBG;
  }  // end of twoSameAxisRots()




  private static BranchGroup twoDiffAxesRots()
  // rotations around different axes cannot usually be swapped 
  {

    // rotations: rotZ --> rotX --> red R
    TransformGroup rot1 = Simple3D.rotX(90);
    rot1.addChild(redR);
    TransformGroup rot2 = Simple3D.rotZ(90);
    rot2.addChild(rot1);
/*
  // or
    TransformGroup rot2 = Simple3D.setRotations(90, 0, 90); 
           // x-, y-, then z- rotation in world coords 
           // best to avoid this, at least when starting out
    rot2.addChild(redR);
*/

    // opp. rotations: rotX --> rotZ --> blue R
    TransformGroup rot3 = Simple3D.rotZ(90);
    rot3.addChild(blueR);
    TransformGroup rot4 = Simple3D.rotX(90);
    rot4.addChild(rot3);

    // build top-level scene
    BranchGroup sceneBG = new BranchGroup();
    sceneBG.addChild(yellowR); 
    sceneBG.addChild(rot2);   // red rotations
    sceneBG.addChild(rot4);   // blue rotations

    return sceneBG;
  }  // end of twoDiffAxesRots()





  // multiple translations, rotations, applied to one TG

  private static BranchGroup twoPosTG()
  // two translations applied to one TG; uses moveBy()
  {
    // pos1 (1,0,3) + (1,0,0) --> red R 
    TransformGroup pos1 = Simple3D.setTranslation(1,0,3);
    pos1.addChild(redR);
    Simple3D.moveBy(pos1, 1,0,0);

    // pos2 (1,0.5,3) + (1,0,3) --> blue R 
    TransformGroup pos2 = Simple3D.setTranslation(1,0,0);
    pos2.addChild(blueR);
    Simple3D.moveBy(pos2, 1,0.5,3);   // up a bit, so visible

    // build top-level scene
    BranchGroup sceneBG = new BranchGroup();
    sceneBG.addChild(yellowR);   // yellow
    sceneBG.addChild(pos1);   // red 
    sceneBG.addChild(pos2);   // blue

    return sceneBG;
  }  // end of twoPosTG()


  private static BranchGroup twoRotTG()
  // two rotations (around the same axis) applied to one TG
  {
    // rot1 40 + 50 --> red R 
    TransformGroup rot1 = Simple3D.rotY(40);
    rot1.addChild(redR);
    Simple3D.rotYBy(rot1, 50);

    // rot2 50 + 50 --> blue R 
    TransformGroup rot2 = Simple3D.rotY(50);
    rot2.addChild(blueR);
    Simple3D.rotYBy(rot2, 50);   // rotate a bit more, so visible

    // build top-level scene
    BranchGroup sceneBG = new BranchGroup();
    sceneBG.addChild(yellowR);   // yellow
    sceneBG.addChild(rot1);   // red 
    sceneBG.addChild(rot2);   // blue

    return sceneBG;
  }  // end of twoRotTG()




  private static BranchGroup posRot1TG()
  /* translation and rotation applied to one TG; 
     Avoid this approach since 'and' is really
     4x4 matrix multiplication. Use a scene graph
     hierarchy instead.
  */
  {
    // tg1 45-rotY '+' (0,0,3) --> red R 
    TransformGroup tg1 = Simple3D.rotY(45);
    Simple3D.moveBy(tg1, 0, 0, 3); 
    tg1.addChild(redR);
    System.out.println("tg1 45-rotY '+' (0,0,3) --> red R");
    Simple3D.printMatrix(tg1);

    // tg2 (0,0,3) '+' 45-rotY --> blue R 
    TransformGroup tg2 = Simple3D.setTranslation(0,0,3);
    Simple3D.rotsBy(tg2, 0, 45, 0); 
/*
    // or use setTransforms to combine into one op.
    TransformGroup tg2 = Simple3D.setTransforms(
                             0,0,3,     // (x,y,z) translation
                             0,45,0,    // rotations
                             1);        // scale
*/
    tg2.addChild(blueR);
    System.out.println("tg2 (0,0,3) '+' 45-rotY --> blue R");
    Simple3D.printMatrix(tg2);

    // build top-level scene
    BranchGroup sceneBG = new BranchGroup();
    sceneBG.addChild(yellowR);   // yellow
    sceneBG.addChild(tg1);   // red 
    sceneBG.addChild(tg2);   // blue

    return sceneBG;
  }  // end of posRot1TG()


  private static BranchGroup axisAngles()
  /*  
       sceneBG ----> pos1TG --> axis-angle rot1TG ---> red R
               |
               ----> pos2TG --> axis-angle rot2TG ---> blue R
  */
  {
    // position the red R   (move --> rotate --> shape)
    TransformGroup rot1TG = Simple3D.setRotationAxis(0,1,1, 90);
                               // (0,1,1) axis at ccw 90 degrees
    rot1TG.addChild(redR);
    TransformGroup pos1TG = Simple3D.setTranslation(0,2,0);  
                               // up the y-axis
    pos1TG.addChild(rot1TG);


    // position the blue R   (move --> rotate --> shape)
    TransformGroup rot2TG = Simple3D.setRotationAxis(0,1,1, 180);
                               // (0,1,1) axis at ccw 180 degrees
    rot2TG.addChild(blueR);
    TransformGroup pos2TG = Simple3D.setTranslation(0,0,1);  
                     // along the z-axis
    pos2TG.addChild(rot2TG);


    // build top-level scene
    BranchGroup sceneBG = new BranchGroup();
    sceneBG.addChild(yellowR);   // yellow
    sceneBG.addChild(pos1TG);  // red
    sceneBG.addChild(pos2TG);  // blue
    return sceneBG;
  }  // end of axisAngles()()


}  // end of RTrans class
