
// Rocket.java
// Andrew Davison, ad@fivedots.coe.psu.ac.th, October 2021

/*
  
*/


import org.jogamp.java3d.*;
import org.jogamp.vecmath.*;
import org.jogamp.java3d.utils.geometry.*;


public class Rocket
{
  private BranchGroup rocketBG = null;


  public Rocket()
  /*
        rocketBG --> pos1TG --> cylinder
            |
            ----> pos2TG --> nose cone
            |
            ----> pos3TG ---> left booster
            |
            ----> pos4TG --> right booster

      The rocket will be standing upright at the origin, with its base
      resting on the floor. 
  */
  {
    // build the shapes
    Cone noseCone = Simple3D.texCone(0.25f, 0.5f, "images/grid.jpg");
    // Cone noseCone = new Cone(0.25f, 0.5f, Simple3D.color("blue"));

    Cylinder cyl = Simple3D.texCylinder(0.25f, 2f, "images/steel1.jpg");
    // Cylinder cyl = new Cylinder(0.25f, 2f, Simple3D.color("yellow"));  

    Cone leftBooster = new Cone(0.25f, 0.5f,
                         Simple3D.color("red"));
    Cone rightBooster = new Cone(0.25f, 0.5f,
                         Simple3D.color("red"));


    // move the cyliner up the y-axis to rest on floor
    TransformGroup pos1TG = Simple3D.setTranslation(0, 1, 0); 
    pos1TG.addChild(cyl); 

    // position the top cone up y-axis to rest on cylinder
    TransformGroup pos2TG = Simple3D.setTranslation(0, 2.25, 0); 
    pos2TG.addChild(noseCone); 

    // move the left booster left along x-axis and up 
    TransformGroup pos3TG = Simple3D.setTranslation(-0.25, 0.24, 0); 
    pos3TG.addChild(leftBooster); 

    // move the right booster right along x-axis and up
    TransformGroup pos4TG = Simple3D.setTranslation(0.25, 0.24, 0); 
    pos4TG.addChild(rightBooster); 


    // group the shapes under their own branchgroup
    rocketBG = Simple3D.createBG();
    rocketBG.addChild(pos1TG);  // cyliner
    rocketBG.addChild(pos2TG);  // top cone
    rocketBG.addChild(pos3TG);  // left booster
    rocketBG.addChild(pos4TG);  // right booster

  }  // end of Rocket()



  public BranchGroup getBG()
  {  return rocketBG;  }


  // -------------------- test rig ---------------------

  public static void main(String[] args)
  /*
        flyingRocket --> orbitTG --> posTG --> rotTG --> rocketBG
                             |        ||
                             ----> rotator
  */
  {
    Simple3D j3d = new Simple3D("Rocket", 0, 1, 10, true, true);

    Rocket r = new Rocket();

    // posTH --> rotTG --> rocket
    TransformGroup rotTG = Simple3D.rotZ(-90);  // cw around z-axis
    rotTG.addChild(r.getBG());   

    // translate so centered at (0,1,-2); up and into the scene 
    TransformGroup posTG = Simple3D.setTranslation(-1.25,1,-2);
    posTG.addChild(rotTG);

    // make the rocket orbit clockwise around y-axis
    TransformGroup orbitTG = Simple3D.orbit(posTG, 4000, -Simple3D.Y_AXIS);
    BranchGroup flyingRocket = Simple3D.createBG(orbitTG);

    // BranchGroup flyingRocket = Simple3D.createBG(posTG);

    j3d.rootBG.addChild(flyingRocket);

    // j3d.rootBG.addChild(r.getBG());
  }  // end of main()


}  // end of Rocket class
