
// TimeBehavior.java
// Andrew Davison, Nov. 2021, ad@fivedots.coe.psu.ac.th

/* 
   Behavior that implements a timer, which is triggered 
   every delay ms. It then calls TimeUpdater.tick().
*/

import java.util.*;
import java.awt.event.*;
import java.awt.AWTEvent;

import org.jogamp.java3d.*;


public class TimeBehavior extends Behavior
{
  private WakeupCondition wakes;
  private int delay;
  private TimeUpdater tu;


  public TimeBehavior(int delay, TimeUpdater tu)
  { 
    this.delay = delay;
    this.tu = tu;
    setSchedulingBounds(Simple3D.INFINITE_BOUNDS);
  }


  public void initialize()
  {  
    WakeupCriterion crits[] = new WakeupCriterion[1];
    crits[0] = new WakeupOnElapsedTime(delay);

    wakes = new WakeupOr(crits);
    wakeupOn(wakes);
  }


  public void processStimulus(Iterator<WakeupCriterion> wCrits)
  { 
    WakeupCriterion wakeUp;

    while (wCrits.hasNext()) {
      wakeUp = wCrits.next();
      if (wakeUp instanceof WakeupOnElapsedTime) {
        tu.tick(delay);
      }
    }
    wakeupOn(wakes);
  } // end of processStimulus()


}  // end of TimeBehavior class
