
/*
  	Java3dTree.java
  
  	Project:		Java3dTree
  	Client:		Java 3D Community
  
  	Project Information:
  		dselman@tornadolabs.com
  		http://www.tornadolabs.com
  
  	@author Daniel Selman: dselman@tornadolabs.com
  
  	Display a Java 3D scenegraph in a Swing Tree control
  
  	@version 1.0
*/


// Authors:
// DCS - dselman@tornadolabs.com

// Change History (YYYY/MM/DD). Newest first.
// ==========================================

// [2000/01/04] - DCS - Added detailed reporting for Object (methods and
// fields)
// [2000/01/04] - DCS - Added detailed reporting for Transform3D
// [2000/01/04] - DCS - Added detailed reporting for Alpha
// [2000/01/04] - DCS - Added detailed reporting for Interpolator

// [1999/12/10] - DCS - Added detailed reporting for ColoringAttributes
// [1999/12/10] - DCS - Added detailed reporting for LineAttributes
// [1999/12/10] - DCS - Added detailed reporting for Material
// [1999/12/10] - DCS - Added detailed reporting for PointAttributes
// [1999/12/10] - DCS - Added detailed reporting for PolygonAttributes
// [1999/12/10] - DCS - Added detailed reporting for RenderingAttributes
// [1999/12/10] - DCS - Added detailed reporting for TexCoordGeneration
// [1999/12/10] - DCS - Added detailed reporting for Texture
// [1999/12/10] - DCS - Added detailed reporting for
// TransparencyAttributes

// [1999/11/17] - DCS - Added detailed reporting for Behavior.

// [1999/10/25] - DCS - Added detailed reporting for AmbientLight.
// [1999/10/25] - DCS - Added detailed reporting for DirectionalLight.
// [1999/10/25] - DCS - Added detailed reporting for PointLight.
// [1999/10/25] - DCS - Added detailed reporting for SpotLight.
// [1999/10/25] - DCS - Added detailed reporting for Light.

// [1999/10/23] - DCS - Added expansion of Link Nodes by following
// SharedGroups.
// [1999/10/23] - DCS - Added reporting of Nodes' parents for non-live
// scenegraphs.

// [1999/10/10] - DCS - Added detailed reporting for Background Node.
// [1999/10/10] - DCS - Added ability to call showGraph multiple
// times.

// [1999/10/09] - DCS - Added check for compiled bit so we don't apply
// capability changes to compiled objects.
// [1999/10/07] - DCS - Fixed bug in Group so it reports CollisionBounds

package com.tornadolabs.j3dtree;


import java.awt.*;
import java.util.*;
import javax.swing.*;
import javax.swing.event.*;
import javax.swing.tree.*;

import org.jogamp.java3d.*;
import org.jogamp.vecmath.*;
import org.jogamp.java3d.utils.universe.*;



public class Java3dTree extends JFrame implements TreeSelectionListener
{
  private final String m_kszLineStyle = "Angled";
  private JEditorPane m_TextPane = null;
  private JSplitPane m_SplitPane = null;
  private VirtualUniverse m_Universe = null;
  private JTree m_Tree = null;
  private final String m_kszPackageName = "com.tornadolabs.j3dtree.";

  private Hashtable m_ObjectInfoTable = null;


  public Java3dTree()
  {
    super("Java 3D Scenegraph");
    buildObjectInfoTable();
  }


  private void buildObjectInfoTable()
  {
    m_ObjectInfoTable = new Hashtable(100);

    addObjectInfoTableItem("org.jogamp.java3d.Alpha");
    addObjectInfoTableItem("org.jogamp.java3d.AmbientLight");
    addObjectInfoTableItem("org.jogamp.java3d.Appearance");
    addObjectInfoTableItem("org.jogamp.java3d.Background");
    addObjectInfoTableItem("org.jogamp.java3d.Behavior");
    addObjectInfoTableItem("org.jogamp.java3d.Bounds");
    addObjectInfoTableItem("org.jogamp.java3d.BoundingLeaf");
    addObjectInfoTableItem("org.jogamp.java3d.BranchGroup");
    addObjectInfoTableItem("org.jogamp.java3d.ColoringAttributes");
    addObjectInfoTableItem("org.jogamp.java3d.Clip");
    addObjectInfoTableItem("org.jogamp.java3d.DirectionalLight");
    addObjectInfoTableItem("org.jogamp.java3d.Fog");
    addObjectInfoTableItem("org.jogamp.java3d.Group");
    addObjectInfoTableItem("org.jogamp.java3d.Interpolator");
    addObjectInfoTableItem("org.jogamp.java3d.Leaf");
    addObjectInfoTableItem("org.jogamp.java3d.Light");
    addObjectInfoTableItem("org.jogamp.java3d.LineAttributes");
    addObjectInfoTableItem("org.jogamp.java3d.Link");
    addObjectInfoTableItem("org.jogamp.java3d.Locale");
    addObjectInfoTableItem("org.jogamp.java3d.Material");
    addObjectInfoTableItem("org.jogamp.java3d.Morph");
    addObjectInfoTableItem("org.jogamp.java3d.OrderedGroup");
    addObjectInfoTableItem("org.jogamp.java3d.PointAttributes");
    addObjectInfoTableItem("org.jogamp.java3d.PointLight");
    addObjectInfoTableItem("org.jogamp.java3d.PolygonAttributes");
    addObjectInfoTableItem("org.jogamp.java3d.utils.geometry.Primitive");
    addObjectInfoTableItem("org.jogamp.java3d.RenderingAttributes");
    addObjectInfoTableItem("org.jogamp.java3d.SceneGraphObject");
    addObjectInfoTableItem("org.jogamp.java3d.Shape3D");
    addObjectInfoTableItem("org.jogamp.java3d.SharedGroup");
    addObjectInfoTableItem("org.jogamp.java3d.utils.universe.SimpleUniverse");
    addObjectInfoTableItem("org.jogamp.java3d.Sound");
    addObjectInfoTableItem("org.jogamp.java3d.Soundscape");
    addObjectInfoTableItem("org.jogamp.java3d.SpotLight");
    addObjectInfoTableItem("org.jogamp.java3d.Switch");
    addObjectInfoTableItem("org.jogamp.java3d.TexCoordGeneration");
    addObjectInfoTableItem("org.jogamp.java3d.Texture");
    addObjectInfoTableItem("org.jogamp.java3d.TextureAttributes");
    addObjectInfoTableItem("org.jogamp.java3d.Transform3D");
    addObjectInfoTableItem("org.jogamp.java3d.TransformGroup");
    addObjectInfoTableItem("org.jogamp.java3d.TransparencyAttributes");
    addObjectInfoTableItem("org.jogamp.java3d.utils.universe.ViewingPlatform");
    addObjectInfoTableItem("org.jogamp.java3d.ViewPlatform");
    addObjectInfoTableItem("org.jogamp.java3d.VirtualUniverse");
  } /* buildObjectInfoTable */



  public Class getClass(String szClass)
  {
    Class classObject = null;
    try {
      classObject = Class.forName(szClass);
    }
    catch(Exception e)
    {
      try  {
        classObject = Class.forName(m_kszPackageName + "ObjectInfo");
      }
      catch(Exception e2)
      {   classObject = null;  }
    }

    return classObject;
  } /* getClass */



  public void setCapabilities(Object obj)
  {
    try  {
      if(obj != null) {
        ObjectInfo objInfo = getObjectInfo(obj.getClass().getName());
        if(objInfo != null)
          objInfo.setCapabilityBits(this, obj);
      }
/*
      else
        System.err.println(
          "Warning: ignored null object in setCapabilities.");
*/
    }
    catch(Exception e)
    {
      System.err.println(
        "Warning: caught Exception applying capabilities: " + e.toString());
    }
  } /* setCapabilities */



  public void recurseObject(Object obj, DefaultMutableTreeNode parent)
  {
    if(obj != null) {
      ObjectInfo objInfo = getObjectInfo(obj.getClass().getName());
      if(objInfo != null)
        objInfo.addToTree(this, parent, obj);
    }
/*
    else
      System.err.println("Warning: ignored null object in recurseObject.");
*/
  } /* recurseObject */



  public ObjectInfo getObjectInfo(String szClass)
  {
    // System.out.println("Looking up: " + szClass);
    ObjectInfo objReturn = (ObjectInfo) m_ObjectInfoTable.get(szClass);
    // System.out.println("Direct Match: " + szClass);

    Class classObject = null;
    try {
      classObject = Class.forName(szClass);
    }
    catch(Exception e)
    { }

    Class parentClass = null;
    if(classObject != null)
      parentClass = classObject.getSuperclass();

    while(objReturn == null && parentClass != null) {
      // if we did not get a match we should walk
      // up the inheritance tree and look for a match on a base class
      objReturn = (ObjectInfo) m_ObjectInfoTable.get(parentClass.getName());
      parentClass = parentClass.getSuperclass();
      // System.out.println("Parent Class: " + parentClass);
    }

    // System.out.println("Returning: " + objReturn);
    return objReturn;
  } /* getObjectInfo */



  private void addObjectInfoTableItem(String szClass)
  {
    // strip the package name from szClass (everything before the final ".")
    int nIndex = szClass.lastIndexOf(".");

    String szTrimedClass = szClass;
    if(nIndex > -1)
      szTrimedClass = szClass.substring(nIndex+1, szClass.length());
    szTrimedClass = m_kszPackageName + szTrimedClass;
    szTrimedClass += "_Info";
    // System.out.println(szClass + "-->" + szTrimedClass);

    Class classObject = getClass(szTrimedClass);
    try {
      if(classObject != null) {
        // m_ObjectInfoTable.put(szClass, classObject.newInstance());
        m_ObjectInfoTable.put(szClass, classObject.getDeclaredConstructor().newInstance());
        return;
      }
    }
    catch(Exception e)
    {   System.err.println(e.toString());  }

    System.err.println(
      "Unable to create a new instance of info class for: " + szClass);
  } /* addObjectInfoTableItem */



  public void valueChanged(TreeSelectionEvent e)
  {
    DefaultMutableTreeNode node =
      (DefaultMutableTreeNode) m_Tree.getLastSelectedPathComponent();
    if (node == null)   // nothing selected
      return;

    Object sceneGraphObject = node.getUserObject();

    // get the _Info object for the node type
    ObjectInfo objInfo = getObjectInfo(sceneGraphObject.getClass().getName());
    try {
      if(objInfo != null) {
        String szText = objInfo.getInfo(sceneGraphObject);
        // System.out.println(szText);
        displayText(szText);
      }
      else
        displayText("No Info Handler for:" + sceneGraphObject);
    }
    catch(Exception ex)
    {   displayText(ex.toString());     }

  } /* valueChanged */



  private void displayText(String szText)
  {   m_TextPane.setText(szText);
      m_TextPane.setCaretPosition(0);
  }


  public void showGraph(VirtualUniverse universe)
  {
    m_Universe = universe;

    //Create the top level parent node.
    DefaultMutableTreeNode top = new DefaultMutableTreeNode("Scenegraph");

    recurseObject(m_Universe, top);

    //Create a tree that allows one selection at a time.
    m_Tree = new JTree(top);
    m_Tree.getSelectionModel().setSelectionMode(
      TreeSelectionModel.SINGLE_TREE_SELECTION);

    // fully expand
    for (int i = 0; i < m_Tree.getRowCount(); i++)
      m_Tree.expandRow(i);

    // collapse third row -- the ViewingPlatform
    m_Tree.collapseRow(3);

    //Listen for when the selection changes.
    m_Tree.addTreeSelectionListener(this);

    //Create the scroll pane and add the tree to it.
    JScrollPane treeView = new JScrollPane(m_Tree);
    Dimension minimumSize = new Dimension(100, 50);
    treeView.setMinimumSize(minimumSize);

    //Add the scroll panes to a split pane.
    if(m_SplitPane == null) {
      //Create the viewing pane.
      m_TextPane = new JEditorPane();
      m_TextPane.setEditable(false);
      JScrollPane htmlView = new JScrollPane(m_TextPane);

      m_SplitPane = new JSplitPane(JSplitPane.VERTICAL_SPLIT);

      m_SplitPane.setTopComponent(treeView);
      m_SplitPane.setBottomComponent(htmlView);

      htmlView.setMinimumSize(minimumSize);
      m_SplitPane.setDividerLocation(500);

      m_SplitPane.setPreferredSize(new Dimension(500, 750));

      //Add the split pane to this frame.
      getContentPane().add(m_SplitPane, BorderLayout.CENTER);

      pack();
      setLocationToTopRight();
      setVisible(true);
    }
    else
      m_SplitPane.setTopComponent(treeView);
  } /* showGraph */



  private void setLocationToTopRight() 
  {
    GraphicsConfiguration config = getGraphicsConfiguration();
    Rectangle bounds = config.getBounds();
    Insets insets = Toolkit.getDefaultToolkit().getScreenInsets(config);

    int x = bounds.x + bounds.width - insets.right - getWidth() - 10;
    int y = bounds.y + insets.top;
    // System.out.println("setLocationToTopRight: " + x + ", " + y);
    setLocation(x, y);
  }

}