
/*
  	ObjectInfo.java
  
  	Project:		Java3dTree
  	Client:		Java 3D Community
  
  	Project Information:
  		dselman@tornadolabs.com
  		http://www.tornadolabs.com
  
  	@author Daniel Selman: dselman@tornadolabs.com
*/

package com.tornadolabs.j3dtree;


import java.lang.reflect.*;
import javax.swing.tree.DefaultMutableTreeNode;
import java.util.*;

import org.jogamp.java3d.*;
import org.jogamp.vecmath.*;
import org.jogamp.java3d.utils.universe.*;



public abstract class ObjectInfo
{
  public ObjectInfo()
  { }


  public String insertSectionBreak(String szText)
  {
    return szText;  
    // return (szText + "\r\n"); // ==============================\r\n");
  }


  public int[] getCapabilityBits()
  {     return null;   }


  private Iterator getChildrenHelper(Object obj)
  {
    Iterator kids = null;
    try {
      kids = getChildren(obj);
    }
    catch(Exception e)
    {   System.err.println(e.toString());   }

    return kids;
  } /* getChildrenHelper */


  public void setCapabilityBits(Java3dTree j3dTree, Object obj)
  {
    int[] nBitsArray = getCapabilityBits();

    if(nBitsArray != null && obj instanceof SceneGraphObject)
      for(int n = 0; n < nBitsArray.length; n++)
        ((SceneGraphObject) obj).setCapability(nBitsArray[n]);

    Iterator kids = getChildrenHelper(obj);
    if(kids != null)
      while(kids.hasNext() == true)
        j3dTree.setCapabilities(kids.next());
  } /* setCapabilityBits */


  public Iterator getChildren(Object obj)
  {  return null;   }


  // Vector + Iterator
  public Iterator combine(Vector v, Iterator kids)
  {
    if(kids != null)
      while(kids.hasNext() == true && v != null)
        v.addElement(kids.next());

    if(v != null)
      return v.iterator();
    else
      return kids;
  } /* combine */


  // Vector + Vector
  public Iterator combine(Vector v1, Vector v2)
  {
    return combine(v1, v2.iterator());
  }


  // Iterator + Iterator
  public Iterator combine(Iterator iter1, Iterator iter2)
  {
    Vector v = new Vector();

    while(iter1 != null && iter1.hasNext() == true)
      v.addElement(iter1.next());

    return combine(v, iter2);
  } /* combine */


  // Iterator + Object
  public Iterator combine(Iterator iter1, Object obj)
  {
    Vector v = new Vector();
    v.addElement(obj);

    while(iter1 != null && iter1.hasNext() == true)
      v.addElement(iter1.next());

    return v.iterator();
  } /* combine */


  public int[] combineArray(int[] a1, int[] a2)
  {
    int[] aRet = null;
    int nTotalLen = 0;
    int nLen1 = 0;
    int nLen2 = 0;

    if(a1 != null){
      nTotalLen += a1.length;
      nLen1 = a1.length;
    }
    if(a2 != null){
      nTotalLen += a2.length;
      nLen2 = a2.length;
    }

    aRet = new int[ nTotalLen ];
    if(a1 != null)
      System.arraycopy(a1, 0, aRet, 0, nLen1);
    if(a2 != null)
      System.arraycopy(a2, 0, aRet, nLen1, a2.length);

    return aRet;
  } /* combineArray */



  public void addToTree(Java3dTree j3dTree,
                         DefaultMutableTreeNode parent,
                         Object obj)
  {
/*
    String objLabel = obj.toString();
    String trimedNm = objLabel;
    if(objLabel.startsWith("org.jogamp.java3d."))
      trimedNm = objLabel.substring("org.jogamp.java3d.".length());

    DefaultMutableTreeNode node = new DefaultMutableTreeNode(trimedNm);
*/
    DefaultMutableTreeNode node = new DefaultMutableTreeNode(obj);
    parent.add(node);

    Iterator kids = getChildrenHelper(obj);
    if(kids != null)
      while(kids.hasNext() == true)
        j3dTree.recurseObject(kids.next(), node);
  } /* addToTree */



  protected String addFields(Class classObj, boolean bPublic)
  {
    String szText = new String();
    Field[] fieldArray = classObj.getDeclaredFields();

    if(fieldArray != null)
      for(int n = 0; n < fieldArray.length; n++) {
        Field field = fieldArray[n];
        int nModifiers = field.getModifiers();
        if(bPublic != false) {
          if(Modifier.isPublic(nModifiers) != false)
            szText += field.getName() + "\r\n";
        }
        else if(Modifier.isProtected(nModifiers) != false)
          szText += field.getName() + "\r\n";
      }

    return szText;
  } /* addFields */



  protected String addMethods(Class classObj, boolean bPublic)
  {
    String szText = new String();
    Method[] methodArray = classObj.getDeclaredMethods();

    if(methodArray != null)
      for(int n = 0; n < methodArray.length; n++) {
        Method method = methodArray[n];
        int nModifiers = method.getModifiers();
        if(bPublic != false) {
          if(Modifier.isPublic(nModifiers) != false)
            szText += method.getName() + "\r\n";
        }
        else if(Modifier.isProtected(nModifiers) != false)
          szText += method.getName() + "\r\n";
      }

    return szText;
  } /* addMethods */


  public String getInfo(Object obj)
  {
    String szText = "";
/*
    szText = insertSectionBreak(szText);

    // szText += "Object\r\n";
    szText += "Type: " + obj.toString() + "\r\n";

    Class classObj = obj.getClass();

    szText += "\r\n";
*/
/*
    szText += "Public Fields\r\n";
    szText += "=============\r\n";
    szText += addFields(classObj, true);

    szText += "\r\n";
    szText += "Protected Fields\r\n";
    szText += "================\r\n";
    szText += addFields(classObj, false);

    szText += "\r\n";
    szText += "Public Methods\r\n";
    szText += "==============\r\n";
    szText += addMethods(classObj, true);

    szText += "\r\n";
    szText += "Protected Methods\r\n";
    szText += "=================\r\n";
    szText += addMethods(classObj, false);
*/
    return szText;
  } /* getInfo */


  public String showMatrix(Transform3D t3d)
  {
     StringBuilder sb = new StringBuilder("Transform ==> \r\n");
     double[] mat = new double[16]; 
     t3d.get(mat);  // stored row-by-row
     for(int row = 0; row < 4; row++){
       sb.append("  | ");
       for(int col = 0; col < 4; col++)
         sb.append(String.format("%5.2f ", mat[row*4 + col]));
       sb.append(" |\r\n");
     }
     sb.append("\r\n");
     return sb.toString();
  }  // end of showMatrix()

}